/* wish_tree.pl
 *
 * Written by Mark Johnson, 18th Febuary 1995
 *
 * Modifications by EPS as indicated
 *
 * Modified by Bryan Jurish, 17th March 2001 -- adapted for SWI prolog on win32
 *
 * A Wish (tk shell) interface for the drawTree tree-drawing package.
 *
 * wish_tree/1 creates a Wish process if necessary, and 
 * sends it commands that make it draw the tree.
 *
 * Example:
 * wish_tree(s/[np/[det/(-every),n1/[adj/(-intelligent),n/(-citizen)]],vp/[v/(-votes),np/(-anarchist)]]).
 *
 * Example with double lines:
 * wish_tree(a/['$m'(b)/[],'$m'(c)/[]]).
 */

:- module(wish_tree, [wish_tree/1]).

%% -- how we start wish (the Tk shell) --
wish_command('c:\\programme\\tcl\\bin\\wish83.exe'). % Win32 / Tcl+Tk 8.3
%wish_command('wish -geometry 700x500-50+50').       % most UNIX systems
%wish_command('wish').                               % generic

%% -- importing into SWI --
:- use_module([draw_tree, font_info]).

%% -- not in SWI --
%% :- use_module([library(charsio)]).

% EPS additions for compatibility with the Quintus charsio.pl
%%%format_to_chars(_,[Term],Chars) :- term_to_chars(Term,Chars).
% for SWI:
format_to_chars(Format,Args,Chars) :- sformat(Chars,Format,Args).

%% -- not needed for SWI ---
%%:- ensure_loaded(library(pipe)).

% END EPS compatibility

:- dynamic wish_streams/2.

% Win32 hack
% - this file is used to store the (temporary) wish-scripts,
%   which on any sensible system would be fed to the running wish
%   process over a pipe.
%   --> THIS FILE GETS REGULARLY OVER-WRITTEN <--
wish_filename('wish_tree_tmp.tk').

% :- op(100, fx, -).	% EPS: keep default setting

wish_tree(Tree) :- draw_tree(Tree, wish_tree:dt), !.

/* Wish interface to drawTree
 */

%wish_font_name('-*-times-medium-r-normal--*-120-*').
%wish_font_name('-*-times-bold-r-normal--*-120-*').
wish_font_name('-*-times-bold-r-normal--*-140-*').

% This parses a term of the form Label/Subtrees as representing a tree
% with label Label and subtrees Subtrees, and a term Label/(-Word)
% as tree with label Label and a single subtree containing a single
% node labelled Word.
%
dt(tree(Label/(-Word), Label, [Word/[]])) :- !.
dt(tree((-Word), Word, [])) :- !.		% EPS: for alternative format
dt(tree(Label/Subtrees, Label, Subtrees)).

%dt(labelsize(Label, PixWidth, 10)) :-
dt(labelsize(Label, PixWidth, 20)) :-
	format_to_chars("~p", [Label], LabelChars),
%	font_width(tmr12, LabelChars, PixWidth).
	font_width(tbr12, LabelChars, PixWidth).

%dt(xgap(10)).
%dt(ygap(10)).

dt(xgap(45)).
dt(ygap(30)).


dt(openstream(Xmax, Ymax, _Tree, WIS)) :-
	wish_filename(WishFile),
	retractall(wish_streams(_,_)),
	open(WishFile, write, WIS), WOS = user,
	assert(wish_streams(WIS,WOS)),

	    % -- SWI complains: "string too long" --
	    format(WIS, "wm title .  ""wish_tree (hack)""~n", []),
	    format(WIS, "wm minsize . 20 20~n", []),
	    format(WIS, "frame .f~n", []),
	    format(WIS, "canvas .c -xscrollcommand {.hscroll set} -yscrollcommand {.vscroll set} -scrollregion {-10p -10p 1000p 1000p} -bg white~n", []),
	    format(WIS, "scrollbar .hscroll -orient horizontal~n", []),
	    format(WIS, "pack .hscroll -side bottom -fill x -in .f~n", []),
	    format(WIS, "pack .c -side top -expand 1 -fill both -in .f~n", []),
	    format(WIS, "scrollbar .vscroll -orient vertical~n", []),
	    format(WIS, "pack .vscroll -side right -fill y~n", []),
	    format(WIS, "pack .f -side left -expand 1 -fill both~n", []),
	    format(WIS, ".vscroll config -command {.c yview}~n", []),
	    format(WIS, ".hscroll config -command {.c xview}~n", []),

	Xmax10 is Xmax+10,
	Ymax10 is Ymax+10,
	format(WIS, ".c config -scrollregion {-10p -10p ~0fp ~0fp}~n",
                    [Xmax10, Ymax10]).

dt(drawline(Stream, Stream, X0, Y0, X1, Y1)) :-
	format(Stream, ".c create line ~0fp ~0fp ~0fp ~0fp~n", [X0,Y0,X1,Y1]).

dt(draw0lines(Stream, Stream, _X0, _Y0, _X1, _Y1)) :- !. % EPS special case

dt(draw2lines(Stream, Stream, X0, Y0, X1, Y1)) :- % EPS special case
	X01 is X0-1, X11 is X1-1, 
	format(Stream, ".c create line ~0fp ~0fp ~0fp ~0fp~n", [X01,Y0,X11,Y1]),
	X02 is X0+2, X12 is X1+2, 
	format(Stream, ".c create line ~0fp ~0fp ~0fp ~0fp~n", [X02,Y0,X12,Y1]).

dt(drawlabel(Stream, Stream, X, Y, Label)) :-
	wish_font_name(FontName),
	%format(Stream, ".c create text ~0fp ~0fp -text {~p} -anchor n -font ""~p""~n",
	format(Stream, ".c create text ~0fp ~0fp -text {~w} -anchor n -font ""~w""~n",
	       [X,Y,Label,FontName]).

%% Win32 Hack
%%dt(closestream(Stream)) :-
%%	flush_output(Stream).
dt(closestream(Stream)) :-
				%wish_filename(WishFile),
				%format(Stream, "bind . <Destroy> {file delete {~w}}~n", [WishFile]),
	flush_output(Stream),
	close(Stream),
	run_wish.

%% Win32 Hack
run_wish :-
	wish_command(WishCmd),
	wish_filename(WishFile),
	sformat(WishCmdLine, "~w ~w", [WishCmd, WishFile]),
	win_exec(WishCmdLine, normal).
				%sformat(WishCmdLine, "~w ~w &", [WishCmd, WishFile]),
				%format(user_error, "Starting wish (Tk shell) as: `~s'~n", [WishCmdLine]),
				%shell(WishCmdLine,_ExitStatus).
	

% skip_chars(Stream) keeps calling peek_char/2 and get0/2 until
% peek_char returns -1 (i.e., no more input).
skip_chars(Stream) :-
	peek_char(Stream, N),
	( N >= 0 -> get0(Stream, _N), skip_chars(Stream) ; true ).
